{***********************************************************************}
{                                                                       }
{  Childwin defines the form used to display a dBASE table in table or  }
{  list form.  The form includes a FieldStructure array documents the   }
{  field structure contained in the file header.  It is used to display }
{  and print the field structure.
{                                                                       }
{  Copyright (c) 2000 Douglas P. Baird - dpbaird@sprynet.com            }
{                                                                       }
{***********************************************************************}

unit Childwin;

interface

uses Windows, Messages, Classes, Graphics, Forms, Controls, StdCtrls, Grids, DBGrids,
  Db, DBTables, DBCtrls, dbvConst, Menus;

type
  FieldStructure = record
    Name: string[32];
    FieldType: char;
    Length: string[3];
    Decimal: string[2];
  end;
  TMDIChild = class(TForm)
    DataSource1: TDataSource;
    Table1: TTable;
    DBGrid1: TDBGrid;
    StringGrid1: TStringGrid;
    DBMemo1: TDBMemo;
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormResize(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure DBGrid1ColEnter(Sender: TObject);
    procedure DBMemo1Exit(Sender: TObject);
    procedure StringGrid1SelectCell(Sender: TObject; ACol, ARow: Integer;
      var CanSelect: Boolean);
    procedure FormCreate(Sender: TObject);
    procedure DBGrid1KeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
  private
    { Private declarations }
    procedure ShowMemo(aRow: Integer);
    procedure GetStructure(var msg: TMessage); message GetStruct;
    function GetTablePath: string;
  public
    { Public declarations }
    FieldCnt: Integer;
    BrowseMode: Boolean;
    EditMode: Boolean;
    ShowDeleted: Boolean;
    TableStructure: array of FieldStructure;
    CallName: string;
    procedure ListFields;
    property TablePath: string read GetTablePath;
  end;

implementation

uses Math, SysUtils, Dialogs, Main;

{$R *.DFM}

procedure TMDIChild.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  SetLength(TableStructure,0);
  if MainForm.MDIChildCount = 1 then begin
    MainForm.SetDbActions(False);
    end;
  {fi}
  Action := caFree;
end;

procedure TMDIChild.FormResize(Sender: TObject);
const
  dHeight = 129;
  dWidth = 265;
var
  aHeight, aWidth: Integer;
  Cntr: Integer;
  sTemp: string;
begin
  DbGrid1.Height := ClientHeight;
  DbGrid1.Width := ClientWidth;
  StringGrid1.Height := ClientHeight;
  StringGrid1.Width := ClientWidth;
  StringGrid1.ColWidths[1] := StringGrid1.Width - StringGrid1.ColWidths[0];  
  aHeight := Min(dHeight,ClientHeight);
  aWidth := Min(dWidth,ClientWidth);
  DBMemo1.Height := aHeight;
  DBMemo1.Width := aWidth;
  DBMemo1.Left := (ClientWidth - aWidth) div 2;
  DBMemo1.Top := (ClientHeight - aHeight) div 2;
  sTemp := TablePath;
  with Canvas do
    if TextWidth(stemp+' ('+CallName+')') > ClientWidth - 155 then begin
      aWidth := TextWidth('... ('+CallName+')');
      if aWidth > ClientWidth - 155 then
        Caption := ' ('+CallName+')'
      else begin
        for Cntr := 1 to Length(sTemp) do
          if TextWidth(Copy(sTemp,1,Cntr)) + aWidth > ClientWidth - 155 then begin
            Caption := Copy(sTemp,1,Cntr-1) + '... ('+CallName+')';
            Break;
            end;
          {fi}
        end;
      {fi}
      end
    else
      Caption := sTemp + ' ('+CallName+')';
    {fi}
  {od with}
end;

procedure TMDIChild.FormActivate(Sender: TObject);
begin
  with MainForm do begin
    SetNav;
    if ShowDeleted then
      mToggleDeleted.Caption := 'Hide Deleted'
    else
      mToggleDeleted.Caption := 'Show Deleted';
    {fi}
    DbBrowse.Checked := BrowseMode;
    DbList.Checked := not BrowseMode;
    FileEdit1.Checked := EditMode
    end;
  {od with}
  ShowMemo(StringGrid1.Row);
end;

procedure TMDIChild.DBGrid1ColEnter(Sender: TObject);
begin
  with DbMemo1 do begin
    DataField := '';
    if (DbGrid1.SelectedField.DataType = ftMemo) and
       (DbGrid1.SelectedField.IsNull = False) then
      DataField := DbGrid1.SelectedField.FieldName;
    {fi}
    Visible := (Length(DataField) > 0);
    end;
  {od with}
end;

procedure TMDIChild.DBMemo1Exit(Sender: TObject);
begin
  DbMemo1.Visible := False;
end;

{***********************************************************************}
{                                                                       }
{  ListFields fills in StringGrid1 with the contents, including field   }
{  names of the current record.                                         }
{                                                                       }
{***********************************************************************}

procedure TMDIChild.ListFields;
var
  Cntr: Integer;
  Temp: string;
begin
  for Cntr := 0 to FieldCnt-1 do begin
    StringGrid1.Cells[0,Cntr] := Table1.Fields[Cntr].FieldName;
    if Table1.Fields[Cntr].IsBlob then begin
      with Table1.FieldDefs.Items[Cntr] do
        if DataType = ftTypedBinary then
          Temp := '[Typed Binary]'
        else if DataType = ftBlob then
          Temp := '[Blob]'
        else
          Temp := '[Memo]';
      {od with}
      if Table1.Fields.Fields[Cntr].AsString > ' '  then
        StringGrid1.Cells[1,Cntr] := UpperCase(Temp)
      else
        StringGrid1.Cells[1,Cntr] := Temp;
      end
    else
      StringGrid1.Cells[1,Cntr] := Table1.Fields[Cntr].AsString;
    {fi}
    end;
  {od for}
end;  { TMDIChild.ListFields }

{***********************************************************************}
{                                                                       }
{  ShowMemo makes the control DbMemo1 visible and displays the contents }
{  of the memo field that has just been entered.                        }
{                                                                       }
{***********************************************************************}

procedure TMDIChild.ShowMemo(aRow: Integer);
begin
  if StringGrid1.Cells[1,aRow] = '[MEMO]' then begin
    DbMemo1.DataField := StringGrid1.Cells[0,aRow];
    DbMemo1.Visible := True;
    end
  else
    DbMemo1.Visible := False;
  {fi}
end;

procedure TMDIChild.StringGrid1SelectCell(Sender: TObject; ACol,
  ARow: Integer; var CanSelect: Boolean);
begin
  ShowMemo(aRow);
end;

{***********************************************************************}
{                                                                       }
{  FormCreate uses a message to call GetStructure that creates the      }
{  TableStructure array.                                                }
{                                                                       }
{***********************************************************************}

procedure TMDIChild.FormCreate(Sender: TObject);
{func/proc}
begin  { TMDIChild.FormCreate }
  PostMessage(Self.Handle,GetStruct,0,0);
  BrowseMode := True;
  EditMode := False;
  FormResize(Sender);
end;  { TMDIChild.FormCreate }

{***********************************************************************}
{                                                                       }
{  GetStructure reads the file header of a XBase table and creates the  }
{  TableStructure array.  It must be executed asyncronously with        }
{  FormCreate to allow table creation                                   }
{                                                                       }
{***********************************************************************}

procedure TMDIChild.GetStructure(var msg: TMessage);
var
  F: file of byte;
  aBuffer: array[0..47] of byte;
  DescrSize: Integer;
  NameSize: Integer;
  TypeOffset: Integer;
  LengthOffset: Integer;
  DecimalOffset: Integer;
  DescrNmbr: Integer;
  DescrStart: Integer;
  Cntr: Integer;
  {func/proc}
  procedure MoveName(aDescr: Integer);
  var
    Cntr: Integer;
    aChar: Char;
  begin  {MoveName}
    TableStructure[aDescr].Name[0] := Char(32);
    for Cntr := 0 to NameSize - 1 do begin
      if aBuffer[Cntr] = 0 then
        aChar := ' '
      else
        aChar := Char(aBuffer[Cntr]);
      {fi}
      TableStructure[aDescr].Name[Cntr+1] := aChar;
      end;
    {od for}
    for Cntr := NameSize+1 to 32 do
      TableStructure[aDescr].Name[Cntr] := ' ';
    {od for}
  end;  { MoveName}
begin  { TMDIChild.GetStructure }
  Table1.Active := False;
  AssignFile(F,Table1.DatabaseName+'\'+Table1.TableName);
  Reset(F);
  BlockRead(F,aBuffer,48);
  if (aBuffer[0] and $07) = 4 then begin
    DescrSize := 48;
    NameSize := 32;
    TypeOffset := 32;
    LengthOffset := 33;
    DecimalOffset := 34;
    DescrStart := 68;
    end
  else begin
    DescrSize := 32;
    NameSize := 11;
    TypeOffset := 11;
    LengthOffset := 16;
    DecimalOffset := 17;
    DescrStart := 32;
    end;
  {fi}
  DescrNmbr := ((256*aBuffer[9] + aBuffer[8]) - DescrStart - 1) div DescrSize;
  SetLength(TableStructure,DescrNmbr);
  Seek(F,DescrStart);
  for Cntr := 0 to DescrNmbr-1 do begin
    BlockRead(F,aBuffer,DescrSize);
    MoveName(Cntr);
    TableStructure[Cntr].FieldType := Char(aBuffer[TypeOffset]);
    TableStructure[Cntr].Length := IntToStr(aBuffer[LengthOffset]);
    if Pos(TableStructure[Cntr].FieldType,'NF') = 0 then
      TableStructure[Cntr].Decimal := ''
    else
      TableStructure[Cntr].Decimal := IntToStr(aBuffer[DecimalOffset]);
    end;
  {od for}
  CloseFile(F);
  Table1.Active := True;
end;  { TMDIChild.GetStructure }

function TMDIChild.GetTablePath: string;
begin  { TMDIChild.GetTablePath }
  Result := Table1.DatabaseName + '\'+ Table1.TableName;
end;  { TMDIChild.GetTablePath }

procedure TMDIChild.DBGrid1KeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if (Key = VK_PRIOR) or (Key = VK_NEXT) or (Key = VK_UP) or
     (Key = VK_DOWN) or (Key = VK_TAB) or (Key = VK_HOME) or
     (Key = VK_END) then
    MainForm.SetNav;
end;

end.

