{***********************************************************************}
{                                                                       }
{  dBASE Viewer provides the ability to:                                }
{                                                                       }
{    * View one or more XBase files supported by the Borland Database   }
{      Engine.                                                          }
{    * Set table properties to;                                         }
{        - Apply filters                                                }
{        - Set index                                                    }
{        - Link to a master file                                        }
{    * Create dBase Level 7 tables that are:                            }
{        - Are new                                                      }
{        - Clone an existing XBase file                                 }
{    * Print table structure for one table or all open tables           }
{                                                                       }
{  Copyright (c) 2000 Douglas P. Baird - dpbaird@sprynet.com            }
{                                                                       }
{  Uses the following components not part of Delphi 5:                  }
{                                                                       }
{    * TDatagrid from DataGrid  EC Software 1997-1999 at               }
{      http://www.ec-software.com also available on Torry's Delphi      }
{      Pages at http://www.torry.webnorth.com/index.htm                 }
{    * TPrintPreview from PrintPrev by Doug Baird and James "Woody"     }
{      Woodard included as part of this project.  A manual and help     }
{      file for this component are available from the author.           }
{                                                                       }
{***********************************************************************}
{                                                                       }
{  Version 1.1 12/5/2000                                                }
{                                                                       }
{  Fixed GetCallName to eliminate duplicates from table names with      }
{        numeric suffixes.                                              }
{  Added facilities to display deleted records and undeleted selected   }
{        records.  Based on tips from www.swissdelphicenter.ch          }
{  Added a facility to pack a table.                                    }
{                                                                       }
{***********************************************************************}
unit Main;

interface

uses Windows, SysUtils, Classes, Graphics, Forms, Controls, Menus,
  StdCtrls, Dialogs, Buttons, Messages, ExtCtrls, ComCtrls, StdActns,
  ActnList, ToolWin, ImgList, Db, DBTables, ChildWin, PrintPrev, dbvConst;

type
  TMainForm = class(TForm)
    { Menu Object }
    MainMenu1: TMainMenu;
      File1: TMenuItem;
        FileNewItem: TMenuItem;
        FileOpenItem: TMenuItem;
        Clone1: TMenuItem;
        FileCloseItem: TMenuItem;
        SaveDesktop1: TMenuItem;
        LoadDesktop1: TMenuItem;
        PrintStructures1: TMenuItem;
        mExit: TMenuItem;
      Database1: TMenuItem;
        Browse1: TMenuItem;
        ListView1: TMenuItem;
        Top1: TMenuItem;
        Prior1: TMenuItem;
        Next1: TMenuItem;
        Last1: TMenuItem;
        Delete1: TMenuItem;
        DbProprties1: TMenuItem;
        Structure1: TMenuItem;
        Edit1: TMenuItem;
        mPackTable: TMenuItem;
      Window1: TMenuItem;
        WindowCascadeItem: TMenuItem;
        WindowTileItem: TMenuItem;
        WindowTileItem2: TMenuItem;
        WindowMinimizeItem: TMenuItem;
        WindowArrangeItem: TMenuItem;
      Help1: TMenuItem;
        HelpContentItem: TMenuItem;
        HelpAboutItem: TMenuItem;
      N1: TMenuItem;
      N2: TMenuItem;
      N3: TMenuItem;
      N4: TMenuItem;
      N5: TMenuItem;
    {Action List }
    ActionList1: TActionList;
      { File menu actions }
        FileNew1: TAction;
        FileExit1: TAction;
        FileOpen1: TAction;
        FileClone1: TAction;
        FileClose1: TWindowClose;
        DeskSave: TAction;
        DeskLoad: TAction;
      { Database menu actions }
        DbBrowse: TAction;
        DbList: TAction;
        RecordFirst: TAction;
        RecordPrior: TAction;
        RecordNext: TAction;
        RecordLast: TAction;
        RecordDelete: TAction;
        RecordUndelete: TAction;
        mUndelete: TMenuItem;
        DbProperties: TAction;
        DbStructure: TAction;
        FileEdit1: TAction;
        mToggleDeleted: TMenuItem;
      { Windows menu actions }
        WindowCascade1: TWindowCascade;
        WindowTileHorizontal1: TWindowTileHorizontal;
        WindowTileVertical1: TWindowTileVertical;
        WindowMinimizeAll1: TWindowMinimizeAll;
        WindowArrangeAll1: TWindowArrange;
        WindowNext1: TAction;     { F6 }
        WindowPrevious1: TAction; { Shift+F6 }
      {Help menu actions }
        HelpAbout1: TAction;
    ToolBar1: TToolBar;
      tbFileExit: TToolButton;
      tbSeparator1: TToolButton;
      tbFileNew: TToolButton;
      tbFileOpen1: TToolButton;
      tbFileClone: TToolButton;
      tbFileClose: TToolButton;
      tbSeparator2: TToolButton;
      tbDbBrowse: TToolButton;
      tbDbList: TToolButton;
      tbSeparator3: TToolButton;
      tbRecordFirst: TToolButton;
      tbRecordPrior: TToolButton;
      tbRecordLast: TToolButton;
      tbRecordNext: TToolButton;
      tbDbProperties: TToolButton;
      tbSeparator4: TToolButton;
      tbDbStructure: TToolButton;
      tbSeparator5: TToolButton;
      tbFileEdit: TToolButton;
      tbDelete: TToolButton;
      tbUndelete: TToolButton;
    { Other objects }
    ImageList1: TImageList;  { List of menu|button images }
    DskTable: TTable;        { Used to save desktop }
    Prn: TPrintPreview;      { used to print table structure(s) }
    OpenDialog: TOpenDialog; { Used to open table & load desktop }
    SaveDialog: TSaveDialog; { Used to create table & save desktop }
    StatusBar: TStatusBar;
    StatDialog: TOpenDialog;
    mSaveStructure: TMenuItem;
    procedure DbBrowseExecute(Sender: TObject);
    procedure DbListExecute(Sender: TObject);
    procedure DbPropertiesExecute(Sender: TObject);
    procedure DbStructureExecute(Sender: TObject);
    procedure DeskLoadExecute(Sender: TObject);
    procedure DeskSaveExecute(Sender: TObject);
    procedure FileClone1Execute(Sender: TObject);
    procedure FileClose1Execute(Sender: TObject);
    procedure FileEdit1Execute(Sender: TObject);
    procedure FileExit1Execute(Sender: TObject);
    procedure FileNew1Execute(Sender: TObject);
    procedure FileOpen1Execute(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure HelpAbout1Execute(Sender: TObject);
    procedure PrintStructures1Click(Sender: TObject);
    procedure RecordFirstExecute(Sender: TObject);
    procedure RecordLastExecute(Sender: TObject);
    procedure RecordNextExecute(Sender: TObject);
    procedure RecordPriorExecute(Sender: TObject);
    procedure WindowArrangeAll1Execute(Sender: TObject);
    procedure WindowNext1Execute(Sender: TObject);
    procedure WindowPrevious1Execute(Sender: TObject);
    procedure helpContentItemClick(Sender: TObject);
    procedure RecordDeleteExecute(Sender: TObject);
    procedure mToggleDeletedClick(Sender: TObject);
    procedure mPackTableClick(Sender: TObject);
    procedure RecordUndeleteExecute(Sender: TObject);
    procedure mSaveStructureClick(Sender: TObject);
  private
    { Private declarations }
    fOut: TextFile;
    procedure BuildNew;
    procedure CloseDup(var msg: TMessage); message CloseDb;
    procedure CreateMDIChild(const Name: string);
    procedure InitialOpen(var msg: TMessage); message OpenDb;
    procedure OpenDbFile;
    function GetCallName(aName: string): string;
  public
    { Public declarations }
    GenNDX: Boolean;       { Set New|Clone to indicate index specified }
    procedure PrintStruct(aChild: TMDIChild; First, Last: Boolean);
    function SaveStruct(aChild: TMDIChild; First, Last: Boolean): Boolean;
    procedure SetDbActions(Enable: Boolean);
    procedure SetNav;
  end;

var
  MainForm: TMainForm;

implementation

{$R *.DFM}

uses About, DbParmForm, NewForm, IndexGen, TableStructure, bde;

procedure TMainForm.BuildNew;
var
  Cntr: Integer;
  sTemp: string;
  aType: string;
  FloatPatch: Boolean;
  FieldCount: Integer;
{func/proc}
  function TableOpen(aName: string): Boolean;
  var
    Cntr: Integer;
  begin
    Result := False;
    for Cntr := 0 to MDIChildCount - 1 do
      with MDIChildren[Cntr] as TMDIChild do
        if aName = TablePath then begin
          Result := True;
          Break;
          end;
        {fi}
      {od with}
    {odFor}
  end;  { TableOpen }
begin  { TMainForm.BuildNew }
  if FormNew.ShowModal = mrOk then begin
    SaveDialog.InitialDir := OpenDialog.InitialDir;
    SaveDialog.FileName := '';
    SaveDialog.DefaultExt := 'dbf';
    SaveDialog.Filter := 'dBase Files (.dbf)|*.DBF';
    SaveDialog.FilterIndex := 1;
    if SaveDialog.Execute then  begin
      if FileExists(SaveDialog.FileName) then begin
        if TableOpen(SaveDialog.FileName) then begin
          ShowMessage('The table ' + Caption + ' is open already ' +
             'and cannot be replaced.');
          Exit;
          end
        else if MessageDlg('The table '+ Caption + ' already exits.  ' +
             'Do you want to overwrite it?',
             mtConfirmation,[mbYes,mbNo],0) = mrYes then begin
          DeleteFile(SaveDialog.FileName);
          end;
        {fi}
        end;
      {fi FileExists}
      CreateMDIChild(SaveDialog.FileName);
      with ActiveMDIChild as TMDIChild do begin
        Table1.DatabaseName := ExtractFileDir(Caption);
        Table1.TableName := ExtractFileName(Caption);
        Table1.TableType := ttDBase;
        Table1.FieldDefs.Clear;
        CallName := GetCallName(ChangeFileExt(Table1.TableName,''));
        Caption := Caption + ' ('+CallName+')';
        FloatPatch := False;
        FieldCount := 0;
        for Cntr := 0 to FormNew.DataGrid1.RowCount - 1 do begin
          sTemp := FormNew.DataGrid1.Cells[0,Cntr];
          aType := FormNew.DataGrid1.Cells[1,Cntr];
          if Length(sTemp) > 0 then begin
            Inc(FieldCount);
            if aType = 'Autoincrement' {'+'} then
                  Table1.FieldDefs.Add(sTemp,ftAutoInc,0,False)
            else if aType = 'Binary' {'B'} then
                  Table1.FieldDefs.Add(sTemp,ftTypedBinary,0,False)
            else if aType = 'Currency' {'N'} then
                  Table1.FieldDefs.Add(sTemp,ftBCD,0,False)
            else if aType = 'Date' {'D'} then
                  Table1.FieldDefs.Add(sTemp,ftDate,0,False)
            else if aType = 'Float' {'F'} then begin
                  Table1.FieldDefs.Add(sTemp,ftString,
                      StrToInt(FormNew.DataGrid1.Cells[3,Cntr]),False);
                  FloatPatch := True;
                  end
            else if aType = 'Integer' {'I'} then
                  Table1.FieldDefs.Add(sTemp,ftInteger,0,False)
            else if aType = 'Logical' {'L'} then
                  Table1.FieldDefs.Add(sTemp,ftBoolean,0,False)
            else if aType = 'Memo' {'M'} then
                  Table1.FieldDefs.Add(sTemp,ftMemo,0,False)
            else if aType = 'Number' {'N'} then begin
                  Table1.FieldDefs.Add(sTemp,ftString,
                      StrToInt(FormNew.DataGrid1.Cells[3,Cntr]),False);
                  FloatPatch := True;
                  end
            else if aType = 'OLE' {'G'} then
                  Table1.FieldDefs.Add(sTemp,ftDBaseOle,0,False)
            else if aType = 'Real64' {'O'} then
                  Table1.FieldDefs.Add(sTemp,ftFloat,0,False)
            else if aType = 'String' {'C'} then
                  Table1.FieldDefs.Add(sTemp,ftString,
                      StrToInt(FormNew.DataGrid1.Cells[3,Cntr]),False)
            else if aType = 'Timestamp' {'@'} then
                  Table1.FieldDefs.Add(sTemp,ftDateTime,0,False)
            else Table1.FieldDefs.Add(sTemp,ftUnknown,0,False);
            end;
          {fi}
          end;
        {od for}
        if GenNDX then
          with formCreateNdx.IndexGrid do begin
            Table1.IndexDefs.Clear;
            for Cntr := 1 to RowCount - 1 do
              if Length(Cells[0,Cntr]) > 0 then
                with Table1.IndexDefs do
                  if Length(Cells[1,Cntr]) > 0 then
                    Add(Cells[0,Cntr],Cells[1,Cntr],[])
                  else
                    Add(Cells[0,Cntr],Cells[2,Cntr],[ixExpression]);
                  {fi}
                {od with Table1...}
              {fi}
            {od for}
            end;
          {od with IndexGrid}
        {fi}
        Table1.CreateTable;
        if FloatPatch then
          aDBaseFile.FixDBase(TablePath,FormNew.DataGrid1);
        {fi}
        Table1.Active := True;
        FieldCnt := FieldCount;
        StringGrid1.RowCount := FieldCnt;
        SetDbActions(True);
        end;
      {od with ActiveMDIChild}
      end;
    {fi}
    end;
  {fi}
end;  { TMainForm.BuildNew }

procedure TMainForm.CloseDup(var msg: TMessage);
begin
  FileClose1Execute(Self);
end;

procedure TMainForm.CreateMDIChild(const Name: string);
var
  Child: TMDIChild;
begin  { TMainForm.CreateMDIChild }
  { create a new MDI child window }
  Child := TMDIChild.Create(Application);
  Child.Caption := Name;
  if FileExists(Name) then begin
    Child.Table1.DatabaseName := ExtractFileDir(Name);
    Child.Table1.TableName := ExtractFileName(Name);
    Child.Table1.Active := True;
    Child.DBGrid1.Refresh;
    end;
  {fi}
end;  { TMainForm.CreateMDIChild }

procedure TMainForm.DbBrowseExecute(Sender: TObject);
begin  { TMainForm.DbBrowseExecute }
  if MDIChildCount > 0 then begin
    DbBrowse.Checked := True;
    DbList.Checked := False;
    with ActiveMDIChild as TMDIChild do begin
      StringGrid1.Visible := False;
      DbGrid1.Visible := True;
      DBGrid1.SetFocus;
      DbGrid1.SelectedIndex := StringGrid1.Row;
      BrowseMode := True;
      end;
    {od with}
    end;
  {fi}
end;  { TMainForm.DbBrowseExecute }

procedure TMainForm.DbListExecute(Sender: TObject);
begin  { TMainForm.DbListExecute }
  if MDIChildCount > 0 then begin
    DbBrowse.Checked := False;
    DbList.Checked := true;
    with ActiveMDIChild as TMDIChild do begin
      StringGrid1.Visible := True;
      DbGrid1.Visible := False;
      StringGrid1.SetFocus;
      ListFields;
      StringGrid1.Row := DbGrid1.SelectedIndex;
      BrowseMode := False;
      end;
    {od with}
    end;
  {fi}
end;  { TMainForm.DbListExecute }

procedure TMainForm.DbPropertiesExecute(Sender: TObject);
begin  { TMainForm.DbPropertiesExecute }
  dbProp.ShowModal
end;  { TMainForm.DbPropertiesExecute }

procedure TMainForm.DbStructureExecute(Sender: TObject);
begin  { TMainForm.DbStructureExecute }
  FormStruct.ShowModal;
end;  { TMainForm.DbStructureExecute }

procedure TMainForm.DeskLoadExecute(Sender: TObject);
var
  SaveDir, SaveExt, SaveFilter: string;
  Cntr: Integer;
  Links: Boolean;
  aSourceName: string;
  aSource: TDataSource;
begin  { TMainForm.DeskLoadExecute }
  if MDIChildCount = 0 then begin
    SaveDir := OpenDialog.InitialDir;
    SaveExt := OpenDialog.DefaultExt;
    SaveFilter := OpenDialog.Filter;
    OpenDialog.InitialDir := OpenDialog.InitialDir;
    OpenDialog.FileName := '';
    OpenDialog.DefaultExt := 'dtp';
    OpenDialog.Filter := 'Desktop Open Files (.dtp)|*.DTP';
    OpenDialog.FilterIndex := 1;
    Links := False;
    if OpenDialog.Execute then begin
      DskTable.DatabaseName := ExtractFileDir(OpenDialog.FileName);
      DskTable.TableName := ExtractFileName(OpenDialog.FileName);
      try
        DskTable.Active := True;
        DskTable.First;
        if DskTable.FieldValues['WinState'] = wsNormal then begin
          MainForm.Top := DskTable.FieldValues['Top'];
          MainForm.Left := DskTable.FieldValues['Left'];
          MainForm.Height := DskTable.FieldValues['Height'];
          MainForm.Width := DskTable.FieldValues['Width'];
          end;
        {fi}
        MainForm.WindowState := DskTable.FieldValues['WinState'];
        DskTable.Next;
        while not DskTable.Eof do begin
          CreateMDIChild(DskTable['TblName']);
          with ActiveMDIChild as TMDIChild do begin
            if DskTable.FieldValues['WinState'] = wsNormal then begin
              Top := DskTable.FieldValues['Top'];
              Left := DskTable.FieldValues['Left'];
              Height := DskTable.FieldValues['Height'];
              Width := DskTable.FieldValues['Width'];
              end;
            {fi}
            WindowState := DskTable.FieldValues['WinState'];
            Table1.Filter := DskTable.FieldByName('Filter').AsString;
            Table1.Filtered := DskTable.FieldValues['Filtered'];
            Table1.IndexName := DskTable.FieldByName('IndexName').AsString;
            Links := (not DskTable.FieldByName('MasterSource').IsNull);
            with Table1.FieldList do begin
              FieldCnt := Count;
              StringGrid1.RowCount := FieldCnt;
              end;
            {od with}
            end;
          {od with}
          DskTable.Next;
          end;
        {od while}
        if Links then begin
          DskTable.First;
          DskTable.Next;
          while not DskTable.Eof do begin
            Cntr := DskTable['MasterForm'];
            if Cntr >= 0 then begin
              with MDIChildren[Cntr] as TMDIChild do begin
                aSourceName := Table1.DatabaseName + '\' + Table1.TableName;
                aSource := DataSource1;
                end;
              {od with}
              if aSourceName = DskTable.FieldByName('MasterSource').AsString then
                with MDIChildren[DskTable.RecordCount - DskTable.RecNo]
                                  as TMDIChild do begin
                  Table1.MasterSource := aSource;
                  Table1.MasterFields := DskTable.FieldByName('MasterField').AsString;
                  end
                {od with}
              else
                ShowMessage('Unable to set master source for DskTable for ' +
                       DskTable['TblName']);
              {fi}
              end;
            {fi Cntr > 0}
            DskTable.Next;
            end;
          {od while}
          end;
        DskTable.Active := False;
        SetDbActions(True);
      except
        ShowMessage('The file does not exist or is busy.')
      end {try/excpet};
      end;
    {fi Open}
    OpenDialog.InitialDir := SaveDir;
    OpenDialog.DefaultExt := SaveExt;
    OpenDialog.Filter := SaveFilter;
    end
  else
    ShowMessage('Desktop not empty.');
  {fi}
end;  { TMainForm.DeskLoadExecute }

procedure TMainForm.DeskSaveExecute(Sender: TObject);
var
  Cntr: Integer;
{func/proc}
  procedure GetSource(aSource: TDataSource);
  var
    Cntr: Integer;
  begin
    for Cntr := 0 to MDIChildCount - 1 do
      with MDIChildren[Cntr] as TMDIChild do
        if DataSource1 = aSource then begin
          with Table1 do
            DskTable.FieldValues['MasterSource'] :=
                  DatabaseName + '\' + TableName;
          {od with}
          DskTable.FieldValues['MasterForm'] := Cntr;
          Break;
          end;
        {fi}
      {od with}
    {od for}
  end;
begin  { TMainForm.DeskSaveExecute }
  if MDIChildCount > 0 then begin
    SaveDialog.InitialDir := OpenDialog.InitialDir;
    SaveDialog.FileName := '';
    SaveDialog.DefaultExt := 'dtp';
    SaveDialog.Filter := 'Desktop Save Files (.dtp)|*.DTP';
    SaveDialog.FilterIndex := 1;
    if SaveDialog.Execute then begin
      DskTable.DatabaseName := ExtractFileDir(SaveDialog.FileName);
      DskTable.TableName := ExtractFileName(SaveDialog.FileName);
      DskTable.CreateTable;
      DskTable.Active := True;
      DskTable.Append;
      DskTable.FieldValues['TblName'] := '';
      DskTable.FieldValues['Top'] := MainForm.Top;
      DskTable.FieldValues['Left'] := MainForm.Left;
      DskTable.FieldValues['Height'] := MainForm.Height;
      DskTable.FieldValues['Width'] := MainForm.Width;
      DskTable.FieldValues['WinState'] := MainForm.WindowState;
      DskTable.FieldValues['Filter'] := '';
      DskTable.FieldValues['Filtered'] := False;
      DskTable.FieldValues['IndexName'] := '';
      DskTable.FieldValues['MasterSource'] := '';
      DskTable.FieldValues['MasterField'] := '';
      DskTable.Post;
      for Cntr := MDIChildCount - 1 downto 0 do
        with MDIChildren[Cntr] as TMDIChild do begin
          DskTable.Append;
          DskTable.FieldValues['TblName'] := Caption;
          DskTable.FieldValues['Top'] := Top;
          DskTable.FieldValues['Left'] := Left;
          DskTable.FieldValues['Height'] := Height;
          DskTable.FieldValues['Width'] := Width;
          DskTable.FieldValues['WinState'] := WindowState;
          DskTable.FieldValues['Filter'] := Table1.Filter;
          DskTable.FieldValues['Filtered'] := Table1.Filtered;
          DskTable.FieldValues['IndexName'] := Table1.IndexName;
          if Table1.MasterSource = nil then begin
            DskTable.FieldValues['MasterSource'] := '';
            DskTable.FieldValues['MasterForm'] := -1;
            end
          else
            GetSource(Table1.MasterSource);
          {fi}
          DskTable.FieldValues['MasterField'] := Table1.MasterFields;
          DskTable.Post;
          end;
        {od with}
      {od for}
      DskTable.Active := False;
      end;
    {fi}
    end
  else
    ShowMessage('Nothing to save');
  {fi}
end;  { TMainForm.DeskSaveExecute }

procedure TMainForm.FileClone1Execute(Sender: TObject);
begin  { TMainForm.FileClone1Execute }
  FormNew.LoadLists(TMDIChild(ActiveMDIChild));
  BuildNew;
end;  { TMainForm.FileClone1Execute }

procedure TMainForm.FileClose1Execute(Sender: TObject);
begin  { TMainForm.FileClose1Execute }
  ActiveMDIChild.Close;
  if MDIChildCount = 1 then begin
    SetDbActions(False);
    end;
  {fi}
end;  { TMainForm.FileClose1Execute }

procedure TMainForm.FileEdit1Execute(Sender: TObject);
begin  { TMainForm.FileEdit1Execute }
  with ActiveMDIChild as TMDIChild do begin
    EditMode := not EditMode;
    ShowDeleted := False;
    mToggleDeleted.Caption := 'S&how Deleted';
    RecordUndelete.Visible := False;
    FileEdit1.Checked := EditMode;
    RecordDelete.Visible := EditMode;
    Table1.Active := False;
    Table1.ReadOnly := not EditMode;
    Table1.Active := True;
    if EditMode then
      DBGrid1.Color := clInfoBk
    else
      DBGrid1.Color := clWindow;
    {fi}
    DBGrid1.ReadOnly := not EditMode;
    end;
  {od with}
  if not DbBrowse.Checked then
    DbBrowseExecute(Sender);
  {fi}
end;  { TMainForm.FileEdit1Execute }

procedure TMainForm.FileExit1Execute(Sender: TObject);
begin  { TMainForm.FileExit1Execute }
  Close;
end;  { TMainForm.FileExit1Execute }

procedure TMainForm.FileNew1Execute(Sender: TObject);
begin
  FormNew.ClearLists;
  BuildNew;
end;

procedure TMainForm.FileOpen1Execute(Sender: TObject);
begin  { TMainForm.FileOpen1Execute }
  if OpenDialog.Execute then begin
    CreateMDIChild(OpenDialog.FileName);
    OpenDialog.InitialDir := ExtractFileDir(OpenDialog.FileName);
    OpenDbFile;
    end;
  {fi}
end;  { TMainForm.FileOpen1Execute }

procedure TMainForm.FormCreate(Sender: TObject);
var
  sTemp: string;
begin  { TMainForm.FormCreate }
  sTemp := ParamStr(1);
  if (Length(sTemp) > 0) and FileExists(sTemp) then begin
    PostMessage(Self.Handle,OpenDb,0,0);
    OpenDialog.InitialDir := ExtractFileDir(sTemp);
    end;
  {fi}
end;  { TMainForm.FormCreate }

procedure TMainForm.HelpAbout1Execute(Sender: TObject);
begin  { TMainForm.HelpAbout1Execute }
  AboutBox.ShowModal;
end;  { TMainForm.HelpAbout1Execute }

procedure TMainForm.InitialOpen(var msg: TMessage);
var
  sTemp: string;
  SrchRec: TSearchRec;
begin  { TMainForm.InitialOpen }
  sTemp := ParamStr(1);
  if (UpperCase(ExtractFileExt(sTemp)) = '.DBF') and FileExists(sTemp) then begin
    {***********************************************************************}
    {  The "Send To" function in Windows Explorer uses the short (8.3)      }
    {  form of file names.  The DBE has problems with that format if the    }
    {  table name has been shortened.  The following code is used to find   }
    {  the long file name for the table.                                    }
    {***********************************************************************}
    FindFirst(sTemp,faAnyFile,SrchRec);
    CreateMDIChild(ExtractFileDir(sTemp)+'\'+ SrchRec.Name);
    OpenDbFile;
    end
  else
    ShowMessage('The table ' + sTemp + ' if not a .dbf file or does not exist');
  {fi}
end;  { TMainForm.InitialOpen }

procedure TMainForm.OpenDbFile;
begin  { TMainForm.OpenDbFile }
  SetDbActions(True);
  with ActiveMDIChild as TMDIChild do begin
    CallName := GetCallName(ChangeFileExt(Table1.TableName,''));
    Caption := Caption + ' ('+CallName+')';
    with Table1.FieldList do begin
      FieldCnt := Count;
      StringGrid1.RowCount := FieldCnt;
      end;
    {od with}
    Table1.First;
    end;
  {od with}
  SetNav;
end;  { TMainForm.OpenDbFile }

{***********************************************************************}
{                                                                       }
{  PrintStruct prints the structure for one table associated with       }
{  aChild TMDIChild form.                                               }
{                                                                       }
{  Parameters:                                                          }
{    First - First table on report - Starts report definition           }
{    Last  - Last table on report - Finishes report definition and      }
{            initiates print preview.                                   }
{                                                                       }
{  Called by:                                                           }
{    TMainForm.PrintStructures1Click                                    }
{    TFormStruct.btnPrintClick in TableStructure.pas                    }
{                                                                       }
{***********************************************************************}

procedure TMainForm.PrintStruct(aChild: TMDIChild; First, Last: Boolean);
var
  Cntr: Integer;
  Temp: Single;
begin  { TMainForm.PrintStruct }
  if Prn = nil then
    Prn := TPrintPreview.Create(Self);
  {fi}
  with Prn, aChild do begin
    if First then begin
      Start;
      AutomaticPaging := True;
      SetMargins(0.5,0.5,0.5,0.5);
      SetFontInformation('Courier New',10,[]);
      CreateColumn(1,0.5,2.5);  { Field or index name}
      CreateColumn(2,3.25,1);   { Field type }
      CreateColumn(3,4.5,1);    { Field Size }
      CreateColumn(4,5.75,1);   { Decimals }
      CreateColumn(5,3.25,3);   { Index value }
      end
    else if LinesLeft < 5 then
      NewPage;
    {fi}
    WriteLine(wtLeft,wtNextY,'Table Name: '+Table1.TableName);
    NewLines(1);
    WriteColumn(1,wtLeft,wtNextY,'Name');
    WriteColumn(2,wtCenter,wtThisY,'Type');
    WriteColumn(3,wtCenter,wtThisY,'Length');
    WriteColumn(4,wtCenter,wtThisY,'Decimal');
    Temp := TheYPosition + LineHeightInches;
    DrawLine(0.5,Temp,8.0,temp,1);
    for Cntr := 0 to High(TableStructure) do
      with TableStructure[Cntr] do begin
        WriteColumn(1,wtleft,wtNextY,Name);
        WriteColumn(2,wtCenter,wtThisY,FieldType);
        WriteColumn(3,wtCenter,wtThisY,Length);
        WriteColumn(4,wtCenter,wtThisy,Decimal);
        end;
      {od with}
    {od for}
    Table1.IndexDefs.Update;
    if Table1.IndexDefs.Count > 0 then begin
      if LinesLeft < 3 then
        NewPage
      else
        NewLines(1);
      {fi}
      WriteColumn(1,wtLeft,wtNextY,'Index Name');
      WriteColumn(5,wtLeft,wtThisY,'Field/Expression');
      Temp := TheYPosition + LineHeightInches;
      DrawLine(0.5,Temp,8.0,temp,1);
      for Cntr := 0 to Table1.IndexDefs.Count - 1 do
        with Table1.IndexDefs[Cntr] do begin
          WriteColumn(1,wtLeft,wtNextY,Name);
          if Length(FieldExpression) > 0 then
            WriteColumn(5,wtLeft,wtThisY,FieldExpression)
          else
            WriteColumn(5,wtLeft,wtThisY,Fields);
          {fi}
          end;
        {od with}
      {od for}
      end;
    {fi}
    if Last then
      Finish
    else
      NewLines(2);
    {fi}
    end;
  {od with}
end;  { TMainForm.PrintStruct }

{***********************************************************************}
{                                                                       }
{  SaveStruct writes the structure for one table associated with        }
{  aChild TMDIChild form to a file.                                     }
{                                                                       }
{  Parameters:                                                          }
{    First - First table on report - Opens the output file              }
{    Last  - Last table on report - Closes the output file              }
{                                                                       }
{  Called by:                                                           }
{    TMainForm.mSaveStructuresClick                                     }
{    TFormStruct.}
{                                                                       }
{***********************************************************************}

function TMainForm.SaveStruct(aChild: TMDIChild; First, Last: Boolean): Boolean;
const
  Tab = #9;
  Tabs = #9#9#9;
var
  Cntr: Integer;
  sTemp: string;
begin  { TMainForm.SaveStruct }
  Result := True;
  with aChild do begin
    if First then
      if StatDialog.Execute then begin
        if FileExists(StatDialog.FileName) then
          if MessageDlg('The file '+ExtractFileName(StatDialog.FileName)+
               ' already exists.  Do you want to replace it?',
               mtConfirmation,[mbYes,mbNo],0) = mrNo then begin
            Result := False;
            Exit;
            end;
          {fi}
        {fi}
        AssignFile(fOut,StatDialog.FileName);
        Rewrite(fOut);
        end
      else begin
        Result := False;
        Exit;
        end;
      {fi}
    {fi}
    WriteLn(fOut,'Table Name: '+Table1.TableName);
    WriteLn(fOut);
    WriteLn(fOut,'Name',Tab,'Type',Tab,'Length',Tab,'Decimal');
    for Cntr := 0 to High(TableStructure) do
      with TableStructure[Cntr] do
        WriteLn(fOut,Name,Tab,FieldType,Tab,Length,Tab,Decimal);
      {od with}
    {od for}
    Table1.IndexDefs.Update;
    if Table1.IndexDefs.Count > 0 then begin
      WriteLn(fOut);
      WriteLn(fOut,'Index Name',Tab,'Field/Expression');
      for Cntr := 0 to Table1.IndexDefs.Count - 1 do
        with Table1.IndexDefs[Cntr] do begin
          sTemp := Name + Tab;
          if Length(FieldExpression) > 0 then
            sTemp := sTemp +FieldExpression
          else
            sTemp := sTemp + Fields;
          {fi}
          WriteLn(fOut,sTemp);
          end;
        {od with}
      {od for}
      end;
    {fi}
    if Last then
      CloseFile(fOut)
    else
      WriteLn(fOut);
    {fi}
    end;
  {od with}
end;  { TMainForm.SaveStruct }

{***********************************************************************}
{                                                                       }
{  PrintStructures1Click prints the structure of all tables currently   }
{  Open.  Calls TMainForm.PrintStruct.                                  }
{                                                                       }
{***********************************************************************}

procedure TMainForm.PrintStructures1Click(Sender: TObject);
var
  Cntr: Integer;
  Limit: Integer;
begin  { TMainForm.PrintStructures1Click }
  Limit := MDIChildCount - 1;
  for Cntr := 0 to Limit do
    PrintStruct(TMDIChild(MDIChildren[Cntr]),Cntr=0,Cntr=Limit);
  {od for}
end;  { TMainForm.PrintStructures1Click }

procedure TMainForm.RecordFirstExecute(Sender: TObject);
begin  { TMainForm.RecordFirstExecute }
  with ActiveMDIChild as TMDIChild do begin
    Table1.First;
    if DbList.Checked then
      ListFields;
    {fi}
    end;
  {od with}
  SetNav;
end;  { TMainForm.RecordFirstExecute }

procedure TMainForm.RecordLastExecute(Sender: TObject);
begin  { TMainForm.RecordLastExecute }
  with ActiveMDIChild as TMDIChild do begin
    Table1.Last;
    if DbList.Checked then
      ListFields;
    {fi}
    end;
  {od with}
  SetNav;
end;  { TMainForm.RecordLastExecute }

procedure TMainForm.RecordNextExecute(Sender: TObject);
begin
  with ActiveMDIChild as TMDIChild do begin
    Table1.Next;
    if DbList.Checked then
      ListFields;
    {fi}
    end;
  {od with}
  SetNav;
end;  { TMainForm.RecordNextExecute }

procedure TMainForm.RecordPriorExecute(Sender: TObject);
begin  { TMainForm.RecordPriorExecute }
  with ActiveMDIChild as TMDIChild do begin
    Table1.Prior;
    if DbList.Checked then
      ListFields;
    {fi}
    end;
  {od with}
  SetNav;
end;  { TMainForm.RecordPriorExecute }

{***********************************************************************}
{                                                                       }
{  SetDbActions sets actions for toolbad buttons and menu items to      }
{  reflect existence of at least one open table.                        }
{                                                                       }
{  Called by:                                                           }
{    TMainForm.BuildNew                                                 }
{    TMainForm.BuildNew                                                 }
{    TMainForm.DeskLoadExecute                                          }
{    TMainForm.FileClose1Execute                                        }
{    TMainForm.OpenDbFile                                               }
{                                                                       }
{***********************************************************************}

procedure TMainForm.SetDbActions(Enable: Boolean);
begin  { TMainForm.SetDbActions }
  FileClose1.Visible := Enable;
  FileClose1.Enabled := Enable;
  RecordFirst.Visible := Enable;
  RecordFirst.Enabled := Enable;
  RecordPrior.Visible := Enable;
  RecordPrior.Enabled := Enable;
  RecordNext.Visible := Enable;
  RecordNext.Enabled := Enable;
  RecordLast.Visible := Enable;
  RecordLast.Enabled := Enable;
  DbProperties.Visible := Enable;
  DbProperties.Enabled := Enable;
  DbStructure.Visible := Enable;
  DbStructure.Enabled := Enable;
  DbBrowse.Visible := Enable;
  DbBrowse.Enabled := Enable;
  DbList.Visible := Enable;
  DbList.Enabled := Enable;
  FileEdit1.Visible := Enable;
  FileEdit1.Enabled := Enable;
  FileClone1.Visible := Enable;
  FileClone1.Enabled := Enable;
  FileClose1.Visible := Enable;
  FileClose1.Enabled := Enable;
  Database1.Visible := Enable;
  Database1.Enabled := Enable;
  SaveDesktop1.Visible := Enable;
  LoadDesktop1.Visible := not Enable;
  with ActiveMDIChild as TMDIChild do 
    RecordDelete.Visible := EditMode and Enable;
  {od with}
end; { TMainForm.SetDbActions }

{***********************************************************************}
{                                                                       }
{  WindowNext1Execute activates the next TMDIChild window.  Executed    }
{  with F6                                                              }
{                                                                       }
{***********************************************************************}

procedure TMainForm.WindowNext1Execute(Sender: TObject);
begin  { TMainForm.WindowNext1Execute }
  Next;
end;  { TMainForm.WindowNext1Execute }

{***********************************************************************}
{                                                                       }
{  WindowPrevious1Execute activates the prior TMDIChild window.         }
{  Executed with Shift+F6                                                     }
{                                                                       }
{***********************************************************************}

procedure TMainForm.WindowPrevious1Execute(Sender: TObject);
begin  { TMainForm.WindowNext1Execute }
  Previous;
end;  { TMainForm.WindowNext1Execute }


procedure TMainForm.WindowArrangeAll1Execute(Sender: TObject);
var
  I: Integer;
begin  { TMainForm.WindowArrangeAll1Execute }
  for I := 0 to MDIChildCount - 1 do
    MDIChildren[I].WindowState := wsNormal;
end;  { TMainForm.WindowArrangeAll1Execute }

{***********************************************************************}
{                                                                       }
{  SetNav sets the enables/disables the toolbar database navigator      }
{  buttons and menu items to refelct table status.                      }
{                                                                       }
{  Called by:                                                           }
{    TMainForm.OpenDbFile                                               }
{    TMainForm.RecordFirstExecute                                       }
{    TMainForm.RecordLastExecute                                        }
{    TMainForm.RecordNextExecute                                        }
{    TMainForm.RecordPriorExecute                                       }
{    TMDIChild.FormActivate in Childwin.pas                             }
{                                                                       }
{***********************************************************************}

procedure TMainForm.SetNav;
begin  { TMainForm.SetNav }
  with ActiveMDIChild as TMDIChild do begin
    RecordLast.Enabled := not Table1.Eof;
    RecordNext.Enabled := not Table1.Eof;
    RecordFirst.Enabled := not Table1.Bof;
    RecordPrior.Enabled := not Table1.Bof;
    end;
  {od with}
end;  { TMainForm.SetNav }

procedure TMainForm.helpContentItemClick(Sender: TObject);
begin
  Application.HelpCommand(HELP_FINDER, 0);
end;

procedure TMainForm.RecordDeleteExecute(Sender: TObject);
begin
  with ActiveMDIChild as TMDIChild do begin
    if MessageDlg('Are you sure that you want to delete this record?',
         mtConfirmation,[mbYes,mbNo],0) = mrYes then
      Table1.Delete;
    {fi}
    end;
  {od with}
end;

{***********************************************************************}
{                                                                       }
{  GetCallName creates a unique identifier for the table being opened   }
{  or created.  It is used when selecting a Master Source in DbParmForm }
{  and is needed to select the correct source when more than one table  }
{  with the same name is open.                                          }
{                                                                       }
{***********************************************************************}
function TMainForm.GetCallName(aName: string): string;
var
  Cntr: Integer;
  sTemp: string;
  aList: TStringList;
{func/proc}
  function GetSuffix(Count: Integer): string;
  begin  { GetSuffix }
    Result := IntToStr(Count);
    {***********************************************************************}
    {  Search for duplicates, incrementing Count until a unique value found.}
    {***********************************************************************}
    while aList.IndexOf(aName+Result) >= 0 do begin
      Inc(Count);
      Result := IntToStr(Count);
      end;
    {od while}
  end;  { GetSuffix }
begin  { TMainForm.GetCallName }
  aList := TStringList.Create;
  for Cntr := 0 to MDIChildCount - 1 do
    with MDIChildren[Cntr] as TMDIChild do begin
      sTemp := Copy(CallName,0,Length(aName));
      if UpperCase(sTemp) = UpperCase(aName) then
        aList.Add(CallName);
      {fi}
      end;
    {od with}
  {od for}
  if aList.Count > 0 then
    Result := aName + GetSuffix(aList.Count)
  else
    Result := aName;
  {fi}
  aList.Free
end;  { TMainForm.GetCallName }

procedure TMainForm.mToggleDeletedClick(Sender: TObject);
begin
  with ActiveMDIChild as TMDIChild do begin
    ShowDeleted := not ShowDeleted;
    RecordUndelete.Visible := ShowDeleted and EditMode;
    if ShowDeleted then
      mToggleDeleted.Caption := '&Hide Deleted'
    else
      mToggleDeleted.Caption := 'S&how Deleted';
    {fi}
    Table1.DisableControls;
    try
      Check(DbiSetProp(hDBIObj(Table1.Handle),curSOFTDELETEON,
            LongInt(ShowDeleted)));
    finally
      Table1.EnableControls;
    end;
    Table1.Refresh;
    end;
  {od with}
end;

procedure TMainForm.mPackTableClick(Sender: TObject);
var
  SaveExcl: Boolean;
begin
  with ActiveMDIChild as TMDIChild do begin
    SaveExcl := Table1.Exclusive;
    if not SaveExcl then begin
      Table1.Active := False;
      try
        Table1.Exclusive := True;
      except
        ShowMessage('Cannot get exclusive use to pack.');
        Exit;
      end;
      Table1.Active := True;
      end;
    {fi}
    Table1.DisableControls;
    Check(DbiPackTable(Table1.DBHandle,Table1.Handle,nil,
          szDBASE,True));
    Table1.EnableControls;
    Table1.Refresh;
    ShowDeleted := False;
    mToggleDeleted.Caption := 'S&how Deleted';
    if not SaveExcl then begin
      Table1.Active := False;
      Table1.Exclusive := False;
      Table1.Active := True;
      end;
    {fi}
    end;
  {od with}
end;

procedure TMainForm.RecordUndeleteExecute(Sender: TObject);
begin
  with ActiveMDIChild as TMDIChild do begin
    Table1.UpdateCursorPos;
    try
      Check(DbiUndeleteRecord(Table1.Handle));
    except
      ShowMessage('No undelete performed.');
    end;
    end;
  {od with}
end;

procedure TMainForm.mSaveStructureClick(Sender: TObject);
var
  Cntr: Integer;
  Limit: Integer;
begin  { TMainForm.mSaveStructuresClick }
  Limit := MDIChildCount - 1;
  for Cntr := 0 to Limit do
    if not SaveStruct(TMDIChild(MDIChildren[Cntr]),Cntr=0,Cntr=Limit) then
      Break;
    {fi}
  {od for}
end;  { TMainForm.mSaveStructuresClick }

end.
